__title__ = 'i3_sway_switch_window'
__description__ = 'In i3wm, or sway, replace current window with emacs buffer, browser tab or other window'
__url__ = 'https://github.com/johanwiden/i3-sway-switch-window'
__version__ = '0.2.0'
__author__ = 'Johan Widén'
__author_email__ = 'j.e.widen@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023 Johan Widén'
